/* This program converts AH data files to SAC
 *
 *			Written by:  Richard Boaz         
 *			Date:  12/08/89      

    This program reads an ah file from standard input and writes a binary
sac file to standard output.  For this reason only one ah file may be processed
at a time; it is therefore necessary to use the ahuncat program to seperate
an ah file with multiple traces into individual files that can then be passed
to this program for processing.  

 */
#include <stdio.h>
#include <rpc/rpc.h>
#include <math.h>
#include "ahhead.h"
#include "sac.h"

char           *progname;
main(argc, argv)
    int             argc;
    char           *argv[];
{
    ahhed           ahhead;
    XDR             xdr_in;
    long            count;
    struct sac      sachead;
    float          *data, *data2, temp;
    void            fill_sac_head();
    int 	    ptr;

    progname = argv[0];
    xdrstdio_create(&xdr_in, stdin, XDR_DECODE);

    if ((out_is_tty() != 0) || (in_is_tty() != 0)) {
	fprintf(stderr, "Usage: %s < stdin > stdout\n", progname);
	exit(-1);
    }

    if (xdr_gethead(&ahhead, &xdr_in) != 1) {
	fprintf(stderr,"Can't read header from ah file.\n");
	exit(-1);
    }


    /* get null sac header */

    sachead = sac_null;

    /* fill sac header */

    fill_sac_head(&ahhead, &sachead);

    /* allocate data space */

    if ((data = (float *) calloc((unsigned) (ahhead.record.ndata), size(&ahhead))) == NULL) {
	fprintf(stderr, "Error allocating space in %s\n", progname);
	exit(-2);
    }

    /* read data */

    switch (ahhead.record.type) {
      case FLOAT:		/* Time Series Data---Stored as FLOAT in ah */
	if (xdr_getdata(&ahhead, data, &xdr_in) == -1) {
	    fprintf(stderr, "Cannot read data from ah file.");
	    exit(-1);
	}
	break;

      case COMPLEX:		/* Spectra---Stored as COMPLEX in ah */
	if (xdr_getdata(&ahhead, data, &xdr_in) == -1) {
	    fprintf(stderr, "Cannot read data from ah file.");
	    exit(-1);
	}
    if ((data2 = (float *) calloc((unsigned) (ahhead.record.ndata), size(&ahhead))) == NULL) {
	fprintf(stderr, "Error allocating space in %s\n", progname);
	exit(-2);
    }

	for (count = 0; count < ahhead.record.ndata; count++) {
	    *(data2+count) = *(data + count * 2);
	}
	ptr = count;
	for (count = 0; count < ahhead.record.ndata; count++) {
	    *(data2+count+ptr) = *(data + count * 2 + 1);
	}
	data = data2;
	break;

      default:
	fprintf(stderr, "Error in %s: %d is an unknown data type\n", progname,
		ahhead.record.type);
	break;
    }

    /* write sac header and data */

	write(1, &sachead, sizeof(struct sac));
	write(1, data, ahhead.record.ndata * size(&ahhead));

    xdr_destroy(&xdr_in);
}

void
fill_sac_head(ahhead, sachead)
    ahhed          *ahhead;
    struct sac     *sachead;
{
    float          *f_pt;
    long           *i_pt, jday;
    char           *c_pt;
    float           e_time;
    double          htoe(), epoch;


    /* fill sacheader */

    sachead->delta = ahhead->record.delta;
    sachead->b = 0;
    sachead->e = ahhead->record.ndata * sachead->delta;

    sachead->depmax = ahhead->record.maxamp;

    /* get event and station location information */

    if (ahhead->station.slat != 0)
	sachead->stla = ahhead->station.slat;
    if (ahhead->station.slon != 0)
	sachead->stlo = ahhead->station.slon;
    if (ahhead->station.elev != 0)
	sachead->stel = ahhead->station.elev;
    if (ahhead->event.lat != 0)
	sachead->evla = ahhead->event.lat;
    if (ahhead->event.lon != 0)
	sachead->evlo = ahhead->event.lon;
    if (ahhead->event.dep != 0)
	sachead->evdp = ahhead->event.dep;

    /* get reference time */

    if (ahhead->record.abstime.yr != 0)
	sachead->nzyear = ahhead->record.abstime.yr;
    if (ahhead->record.abstime.mo != 0 && ahhead->record.abstime.day != 0)
	sachead->nzjday = julian(&ahhead->record.abstime);
    if (ahhead->record.abstime.hr != 0)
	sachead->nzhour = ahhead->record.abstime.hr;
    if (ahhead->record.abstime.mn != 0)
	sachead->nzmin = ahhead->record.abstime.mn;
    if (ahhead->record.abstime.sec != 0) {
	sachead->nzsec = (int) ahhead->record.abstime.sec;
	sachead->nzmsec = (ahhead->record.abstime.sec - (int) ahhead->record.abstime.sec) * 1000;
    }

    /* get number of data points */

    sachead->npts = ahhead->record.ndata;

    /* get data type */

    switch (ahhead->record.type) {
      case FLOAT:
	sachead->iftype = ITIME;
	sachead->iztype = IB;
	break;
      case COMPLEX:
	sachead->iftype = IRLIM;
	break;
      case VECTOR:
	printf("SAC unable to handle VECTOR data.  aborting.\n");
	exit(-1);
      case TENSOR:
	printf("SAC unable to handle TENSOR data.  aborting.\n");
	exit(-1);
      case DOUBLE:
	printf("SAC unable to handle DOUBLE data.  aborting.\n");
	exit(-1);
      default:
	fprintf(stderr, "%d is an unknown data type.  aborting.\n",
		ahhead->record.type);
	exit(-1);
    }

    /* station name */

    strcpy(sachead->kstnm, ahhead->station.code);

    /* component name */

    strcpy(sachead->kcmpnm, ahhead->station.chan);

    /* instrument type */

    strcpy(sachead->kinst, ahhead->station.stype);

    /* various other fields  */

    sachead->internal4 = 6;
    sachead->internal5 = 0;
    sachead->internal6 = 0;
    sachead->leven = TRUE;
    sachead->lpspol = FALSE;
    sachead->lcalda = TRUE;
    sachead->unused27 = FALSE;
}
