/*
	Program SEGYMOD

	Program to modify SEGY headers to add additional information
	not available when the files were created.

	Called by segymod - (options) files
	Available options
	    -f scf cuf	    Set scale flags
			    scf - scalar which should be multiplied times
				all coordinates scf = 1,10,100, etc
				scf negative means divide
				default = 1
			    cuf - coordinate flag 
				= 1 information in meters
				= 2 information in seconds of arc
				default = 2
			    elf - elevation scalar which should be 
				multiplied times the elevations	    
	
	    -r rlong rlat relv   Set receiver locations
			    rlong = receiver longitude or X coord
			    rlat = receiver latitude or Y coord
			    relv = receiver elevation 
			    default ( 0,0,0 )

	    -s slong slat selv   Set source locations
			    slong = source longitude or X coord
			    slat = source latitude or Y coord
			    selv = source elevation
			    default (0,0,0)

		-y year		set year in records
				year = year for both start of record and event

		-g gain   set gain for instrument
				gain = gain

	    -i		    Set interactive mode
			    if this flag set then for each file the program
			    asks for the information for the other set flags
			    after the -i any other flags must be specified
			    i.e. -irf  or -irs .
	    
	Any information requested to be changed but not specified will be set
	to the default

	The SEGY header is standard except as follows:
			trhdr[100] = MSB of sample rate in microseconds
			trhdr[101] = LSB of sample rate in microseconds
			trhdr[102]=1 ==> 32 bit integer data
			trhdr[102]=0 ==> 16 bit integer data
			trhdr[103] = start of trace in milliseconds
			trhdr[104] = trigger time year
			trhdr[105] = trigger time julian day
			trhdr[106] = trigger time hour
			trhdr[107] = trigger time minutes
			trhdr[108] = trigger time seconds
			trhdr[109] = trigger time  milliseconds
			trhdr[110] = MSW of scale factor
			trhdr[111] = LSW of scale factor 
					(amplitude = data * scale/gain)

	Written by J. C. Fowler
	    27 December 1988
*/


#include <stdio.h>	/* Standard I/O header file			*/
#include <ctype.h>
#include <string.h>
#include <fcntl.h>



char *progname;
main (argc, argv)


int		argc;
char		*argv[];

{
   FILE	*fpin;		/* Pointer to the input file */
   
   int i, k, j, int_hold, num_sam, sam_rate, ifile, pos;
   short trhdr[120], short_hold;
    short scf, cuf, elf, year, gain ;
    int  fflag, sflag, rflag, iflag, yflag, gflag ;
    long  rlong, rlat, slong, slat, relv, selv ;     
    long    ltemp[10], *lptr ;
   unsigned char *cptr;
   unsigned char FileIN[64] , FileOUT[64], Dum[8];
   float dt, *data, *dout;

   /* take command line arguments as filesnames and flags or
      ask for the filename to be used for the conversion      */

   progname = argv[0];

   for (i=0 ; i<64 ; i++) FileIN[i] = 0;
   if ( argc == 1 ){
      printf("\n Usage %s file_name(s) \n",progname);
      exit(1); 
   }

/*	set default parameters */
    scf = 1 ;	    /* scale factor for coordinates */
    cuf = 2 ;	    /* coordinate unit flag = 2 lat/long */
    elf = 1 ;	    /* scale factor for elevations */
    slong = 0 ;	    /* source longitude */
    slat = 0 ;      /* source latitude */
    selv = 0 ;	    /* source elevation */
    rlong = 0 ;     /* receiver longitude */
    rlat = 0 ;      /* receiver latitude */
    relv = 0 ;	    /* receiver elevation */
	year = 1989 ;   /* year */
	gain = 1 ;		/* default gain */
    fflag = 0 ;	    /* f flag */
    sflag = 0 ;	    /* s flag */
    rflag = 0 ;	    /* r flag */
    iflag = 0 ;	    /* i flag */
	yflag = 0 ;		/* y flag */
	gflag = 0 ;		/* gain flag */


while ( argc > 1 && argv[1][0] == '-')
{
    switch (argv[1][1])
    {
	case 'f' :	/* -f set flags */
	    fflag = 1 ;	    /* set flag flag */

		/* get data entries */
		sscanf(argv[2],"%d",&k) ;  /* get scale factor */
		scf = k ;
		argv++ ;
		argc-- ;

		sscanf(argv[2],"%d",&k) ;   /* get coordinate flag */
		cuf = k ;
		argv++ ;
		argc-- ;

		sscanf(argv[2],"%d",&k) ;   /* get elevation flag */
		elf = k ;
		argv++ ;
		argc-- ;

	    break ;
	case 's' :	/* -s source changes */
	    sflag = 1 ;	    /* set source flag */
	    
		sscanf(argv[2],"%ld",&slong) ;   /* get source longitude */
		argv++ ;
		argc-- ;
	   
		sscanf(argv[2],"%ld",&slat) ;   /* get source latitude */
		argv++ ;
		argc-- ;
	    
		sscanf(argv[2],"%ld",&selv) ;   /* get source elevation */
		argv++ ;
		argc-- ;
	    
	    break ;
	case 'r' :
	    rflag = 1 ;	    /* set receiver flag */
	    
		sscanf(argv[2],"%ld",&rlong) ;   /* get receiver longitude */
		argv++ ;
		argc-- ;
	    
		sscanf(argv[2],"%ld",&rlat) ;   /* get receiver latitude */
		argv++ ;
		argc-- ;
	    
		sscanf(argv[2],"%ld",&relv) ;   /* get receiver elevation */
		argv++ ;
		argc-- ;
	    
	    break ;
	case 'y' :
	    yflag = 1 ;	    /* set year flag */
	    
		sscanf(argv[2],"%d",&k) ;   /* get year */
		year = k ;
		argv++ ;
		argc-- ;
	    
	    break ;
	case 'g' :
	    gflag = 1 ;	    /* set gain flag */
	    
		sscanf(argv[2],"%d",&k) ;   /* get gain */
		gain = k ;
		argv++ ;
		argc-- ;
	    
	    break ;
	case 'i' :
	    iflag = 1 ;	    /* set interactive flag */
		/* see if any other flags set */
		i = 2 ;
		while( argv[1][i] != '\0')
		{
			switch(argv[1][i])
			{
			case 'f' :
				fflag = 1 ;   /* set f flag */
				break ;	
			case 's' :
				sflag = 1 ;   /* set source flag */
				break ;
			case 'r' :
				rflag = 1 ;		/* set receiver flag */
				break ;
			case 'y' :
				yflag = 1 ;     /* set year flag */
				break ;
			case 'g' :
				gflag = 1 ;     /* set gain flag */
				break ;
			}
			i++ ;
		}
	    break ;
    }
    argc-- ;
    argv++ ;
}

for(ifile=1; ifile < argc; ifile++){
      strcpy( FileIN, &argv[ifile][0] );
      printf("\nINPUT FILE  %s \n",FileIN);

         /* open the input file */

   if (( fpin = fopen(FileIN, "r+")) == NULL ){
      printf( "\n ******  there has been an error opening %s for input  *****", FileIN);
      fclose(fpin);
      exit(1);
   }


   /*  first read the SEGY trace header    */

   cptr = ( unsigned char *) &trhdr[0];
       for ( i= 0; i< 240 ; i++){
      *cptr = fgetc(fpin);
      cptr++;
   }

/*  start processing this header */
/*  see if need to set flags */
    if(fflag == 1)
    {
	/*  yes set flags in header */
	if(iflag == 1)   /* see if interactive mode */
	{
	    /* yes interactive mode print current values */
	    printf("\nInput Flags (S-scalar Coord-Units El-scalar)") ;
	    printf("\nDefault values %d %d %d",scf,cuf,elf) ;
	    
	    /* now get new values if needed */
	    ltemp[0] = (long)scf ;
	    ltemp[1] = (long)cuf ;
	    ltemp[2] = (long)elf ;
	    lptr = &ltemp[0] ;
	    
	    getvalues(3,lptr) ;
	    scf = (short)ltemp[0] ;
	    cuf = (short)ltemp[1] ;
	    elf = (short)ltemp[2] ; 
	}
	/* insert new values in header */
	trhdr[35] = scf ;   /* scalar for coordinate positions */
	trhdr[44] = cuf ;   /* coordinate type flag */
	trhdr[34] = elf ;   /* scalar for elevations */
    }

/* see if need to set source information   */
    if(sflag == 1)
    {
	/* yes set source information in header */
	if(iflag == 1)   /* see if interactive mode */
	{
	    /* yes interactive mode print current values */
	    printf("\nInput Source Position (Slong Slat Selv)") ;
	    printf("\nDefault values %ld %ld %ld",slong,slat,selv) ;
	    
	    /* now get new values if needed */
	    ltemp[0] = slong ;
	    ltemp[1] = slat ;
	    ltemp[2] = selv ;
	    lptr = &ltemp[0] ;
	    
	    getvalues(3,lptr) ;
	    slong = ltemp[0] ;
	    slat = ltemp[1] ;
	    selv = ltemp[2] ; 
	}
	/* insert new values in header */
	hputl(&slong,&trhdr[36]) ;
	hputl(&slat,&trhdr[38]) ;
	hputl(&selv,&trhdr[24]) ;
    }
/* see if need to set receiver information   */
    if(rflag == 1)
    {
	/* yes set receiver information in header */
	if(iflag == 1)   /* see if interactive mode */
	{
	    /* yes interactive mode print current values */
	    printf("\nInput Receiver Position (Rlong Rlat Relv)") ;
	    printf("\nDefault values %ld %ld %ld",rlong,rlat,relv) ;
	    
	    /* now get new values if needed */
	    ltemp[0] = rlong ;
	    ltemp[1] = rlat ;
	    ltemp[2] = relv ;
	    lptr = &ltemp[0] ;
	    
	    getvalues(3,lptr) ;
	    rlong = ltemp[0] ;
	    rlat = ltemp[1] ;
	    relv = ltemp[2] ; 
	}
	/* insert new values in header */
	hputl(&rlong,&trhdr[40]) ;
	hputl(&rlat,&trhdr[42]) ;
	hputl(&relv,&trhdr[20]) ;
    }

/* see if need to set year   */
    if(yflag == 1)
    {
	/* yes set year information in header */
	if(iflag == 1)   /* see if interactive mode */
	{
	    /* yes interactive mode print current values */
	    printf("\nInput year ") ;
	    printf("\nDefault values %d ",year) ;
	    
	    /* now get new values if needed */
	    ltemp[0] = (long)year ;
	    lptr = &ltemp[0] ;
	    
	    getvalues(1,lptr) ;
	    year  = (short)ltemp[0] ;
	}
	/* insert new values in header */
		trhdr[78] = year ;
		trhdr[104] = year ;
	}

/* see if need to set gain   */
    if(gflag == 1)
    {
	/* yes set gain information in header */
	if(iflag == 1)   /* see if interactive mode */
	{
	    /* yes interactive mode print current values */
	    printf("\nInput gain ") ;
	    printf("\nDefault values %d ",gain) ;
	    
	    /* now get new values if needed */
	    ltemp[0] = (long)gain ;
	    lptr = &ltemp[0] ;
	    
	    getvalues(1,lptr) ;
	    gain  = (short)ltemp[0] ;
	}
	/* insert new values in header */
		trhdr[60] = gain ;
	}
/*   rewrite header to file */

    if(fseek(fpin,0L,0) == -1)
    {
	printf("\nSEEK Error\n") ;
	exit(1) ;
    }
    cptr = (unsigned char *)&trhdr[0] ;


    for( i=0 ; i<240 ; i++)
    {
	fputc(*cptr,fpin) ;
	cptr++ ;
    }
   fclose(fpin);

}
}

/********************************************************************/
/*               getvalues(nv,*lptr)                                */
/********************************************************************/
/*	Routine to get long integer values from keyboard
	nv = maximum number of values to get
	*lptr = pointer to array to store values
	    if less than nv values typed in then only change values
	    which were typed in 
	    Space is used as delimiter between values
 */

getvalues(nv,lptr)
int	nv ;
long	*lptr ;
{
    char    dumm[64], *cptr, c ;
    int	    i, inv, ifl ;

    inv = 0 ;	/* initialize value count */
    ifl = 0 ;	/* set flag */
    cptr = dumm ;  /* initialize counter */
    printf("\nNew Values ? ") ;

    while( (c=getchar()) != '\n')   /* check for NL */
    {
	*cptr = c ;	/* put char in buffer */
	if(c != ' ')	/* see if not a space */
	{
	    ifl++ ;	/* set flag for nonblank entry */
	    cptr++ ;	/* increment buffer count */
	}
	else if((ifl != 0) && (inv < nv))  /* see if need to convert */
	{
	    *cptr = '\0' ;  /* yes terminate string */
	    *lptr = atol(dumm) ;  /* convert integer */
	    cptr = &dumm[0] ;	    /* reset counters */
	    ifl = 0 ;
	    inv++ ;
	    lptr++ ;
	}
    }
    if((ifl != 0) && (inv < nv))    /* see if need final value */
    {
	*cptr = '\0' ;	    /* yes convert it */
	*lptr = atol(dumm) ;
    }
    return ;	
}

/***********************************************************************/
/*             hputl(lptr,sptr)                                      */
/***********************************************************************/
/*	Routine to put a long variable in an array of short variables
	sptr = pointer to first position in short array
	lptr = pointer to long variable

 */

hputl(lptr,sptr)
short	*sptr ;
long	*lptr ;
{
    
    short   *sptr1, i ;

    sptr1 = (short *)lptr ;
    for( i=0 ; i<2 ; i++,sptr++,sptr1++) *sptr = *sptr1 ;

    return ;
}
