/*
	Program SEGY2sac

	Program takes modified SEGy files as produced by 
	pas2SEGY program from PASSCAL data on EXABYTE tapes
	and reformats them into a SAC file.

	The SEGY modification is as follows:
		if the sample rate ( in microsecs) is too long to fit
		into the two byte SEGY field in bytes 117 and 118
		then these bytes are set to x '0001' and the actual
		sample rate is placed in bytes 201-204 in the SEGY 
		header
                Also bytes 205 & 206 indicate whether the integer
		data is 16 bit or 32 bit as follows:
			trhdr[102]=1 ==> 32 bit integer data
			trhdr[102]=0 ==> 16 bit integer data


	Written by Tim Ahern, IRIS
		June 10, 1988
*/


#include <stdio.h>	/* Standard I/O header file			*/
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include "sac.h"



main (argc, argv)


int		argc;
char		*argv[];

{
   FILE	*fpin;			/* Pointer to the input file  */

   
   int i, j, int_hold, num_sam, sam_rate;
   short trhdr[120], short_hold;
   unsigned char *cptr;
   unsigned char FileIN[64] ;
   float dt, ar[10000]; 

   /* take command line arguments as filesnames and flags or
      ask for the filename to be used for the conversion      */



   for (i=0 ; i<64 ; i++) FileIN[i] = 0;
   if ( argc > 1 ){
      strcpy( FileIN, &argv[1][0] );
   }else{

   /* now if we didnt get what we needed from the command 
		line we have to ask for it */

      printf ( "\n\nEnter the name of the file to be processed -> " );
      scanf ( "%24s", FileIN );
      if (( fpin = fopen(FileIN, "r")) == NULL ){
         printf( "\n ******  there has been an error opening %s for input  *****",FileIN);
         fclose(fpin);
         exit(1);
      }
      fclose(fpin); /* will be opened again at the usual time */
   }

         /* open the input file and go to the output directory	*/

   if (( fpin = fopen(FileIN, "rb")) == NULL ){
      printf( "\n ******  there has been an error opening %s for input  *****", FileIN);
      fclose(fpin);
      exit(1);
   }

   /*  start reading the data file   */
   /*  first read the SEGY trace header    */

   cptr = ( unsigned char *) &trhdr[0];
   for ( i= 0; i< 240 ; i++){
      *cptr = fgetc(fpin);
      cptr++;
   }
   num_sam = trhdr[57];
   if (trhdr[58] == 1){
      sam_rate = trhdr[100] * 32768 + trhdr[101];
   }else{
      sam_rate = trhdr[58];
   }
   dt = ( float) sam_rate / 1000000.0;

   printf( "\n Number of samples = %10d sample rate (microsecs)= %10d \n",num_sam,sam_rate);
   printf( "\n Sample rate as floating %20f10 \n",dt); 
   /* now go get the trace data */ 
   if (trhdr[102] == 1){            /* 32 bit integer data */

      for ( i=0; i<num_sam; i++){
         cptr = (unsigned char *) &int_hold;
         for (j=0; j<4; j++,cptr++){ 
            *cptr = fgetc(fpin);
         }
         ar[i] = (float) int_hold;
      }
   }else{                         /* 16 bit integer data */
      for ( i=0; i < num_sam; i++){
         cptr = (unsigned char *) &short_hold;
         *cptr = fgetc(fpin);
	 cptr++;
         *cptr = fgetc(fpin);
	 ar[i] = (float) short_hold;
      }
   }
   wrtsac(FileIN,dt,num_sam,ar,1);
   fclose(fpin);
}
   


/*********************************************************************/
/*                   wrtsac (FO,dt,ns,ar,flg)                        */
/*********************************************************************/
/*          Routine to write SAC files                               */
/*		FO = output file name                                */
/*		dt = sample rate in samples per second               */
/*		ns = number of data samples                          */
/*		ar = data array                                      */
/*		flg = output flag                                    */
/*		    = 0 output in binary format                      */
/*		    = 1 output in asci format                        */
/*                                                                   */
/*	    Program will add ".sac" or ".asc" extension to output    */
/*	    File name					             */

wrtsac (FO,dt,ns,ar,flg)
char	FO[] ;
int	ns, flg ;
float	dt, ar[] ;
{
    
        struct  sac	sp ;
	int	j, n0 ;
	float	zero = 0.0 ;

	FILE	*iopo ;


	if (flg == 0)
	{
	    strcat (FO,".sac") ;
	}
	else
	{
	    strcat (FO,".asc") ;
	}
	if((iopo = fopen(FO,"w")) == NULL)
	{
	    printf("cannot open output file %s\n",FO) ;
	    exit (1) ;
	}
    

/* convert parameters to sac header parameters */
	sp = sac_null;

/* floats */
	sp.delta = 1.0/dt ;
	sp.b = 0.0 ;
	sp.e = ns * sp.delta ;

/* integers */
	sp.internal4 = 6;	/* LLL sets it */
	sp.internal5 = 0;	/* LLL sets it */
	sp.internal6 = 0;	/* LLL sets it */
	sp.npts = ns ;
	sp.iftype = ITIME ;
	sp.iztype = IB ;

/* logicals  */
	sp.leven = TRUE ;
	sp.lpspol = FALSE ;     /* LLL sets it */
	sp.lcalda = TRUE ;      /* LLL sets it */
	sp.unused27 = FALSE ;   /* LLL sets it */


	if (flg == 0) 
	{

	/* write binary sac header */
		if (fwrite(&sp, sizeof(struct sac), 1, iopo)!=1) 
		{
			printf("FATAL ERROR: bad  write %s, header\n",FO);
			exit(4);
		}

	/* write floating point data */
		if (fwrite(&ar[0],ns*sizeof(float),1,iopo)!=1) 
		{
			printf("FATAL ERROR: bad  write %s, data\n",FO);
			exit(5);
		}

	/* zero pad to 512 byte blocks (128 4 byte floats) */
		n0 = 128-(sizeof(struct sac)/4+ns)%128;
		for (j=0; j<n0; j++) 
		{
			if (fwrite(&zero, sizeof(float), 1, iopo)!=1) 
			{
				printf("FATAL ERROR: bad  write %s, padding\n"
					,FO);
				exit(6);
			}
		}
	}
    	else 
	{

	/* print header in ascii */
/*		fprintf(iopo, FCS, sp.delta, sp.depmin, sp.depmax, sp.scale, 
			sp.odelta);
		fprintf(iopo, FCS, sp.b, sp.e, sp.o, sp.a, sp.internal1);
		fprintf(iopo, FCS, sp.t0, sp.t1, sp.t2, sp.t3, sp.t4);
		fprintf(iopo, FCS, sp.t5, sp.t6, sp.t7, sp.t8, sp.t9);
		fprintf(iopo, FCS, sp.f, sp.resp0, sp.resp1, sp.resp2, 
			sp.resp3);
		fprintf(iopo, FCS, sp.resp4, sp.resp5, sp.resp6, sp.resp7, 
			sp.resp8);
		fprintf(iopo, FCS, sp.resp9, sp.stla, sp.stlo, sp.stel, 
			sp.stdp);
		fprintf(iopo, FCS, sp.evla, sp.evlo, sp.evel, sp.evdp, 
			sp.unused1);
		fprintf(iopo, FCS, sp.user0, sp.user1, sp.user2, sp.user3, 
			sp.user4);
		fprintf(iopo, FCS, sp.user5, sp.user6, sp.user7, sp.user8, 
			sp.user9);
		fprintf(iopo, FCS, sp.dist, sp.az, sp.baz, sp.gcarc, 
			sp.internal2);
		fprintf(iopo, FCS, sp.internal3, sp.depmen, sp.cmpaz, 
			sp.cmpinc, sp.unused2);
		fprintf(iopo, FCS, sp.unused3, sp.unused4, sp.unused5, 
			sp.unused6, sp.unused7);
		fprintf(iopo, FCS, sp.unused8, sp.unused9, sp.unused10, 
			sp.unused11, sp.unused12);
		fprintf(iopo, ICS, sp.nzyear, sp.nzjday, sp.nzhour, sp.nzmin, 
			sp.nzsec);
		fprintf(iopo, ICS, sp.nzmsec, sp.internal4, sp.internal5, 
			sp.internal6, sp.npts);
		fprintf(iopo, ICS, sp.internal7, sp.internal8, sp.unused13, 
			sp.unused14, sp.unused15);
		fprintf(iopo, ICS, sp.iftype, sp.idep, sp.iztype, sp.unused16,
			sp.iinst);
		fprintf(iopo, ICS, sp.istreg, sp.ievreg, sp.ievtyp, sp.iqual, 
			sp.isynth);
		fprintf(iopo, ICS, sp.unused17, sp.unused18, sp.unused19, 
			sp.unused20, sp.unused21);
		fprintf(iopo, ICS, sp.unused22, sp.unused23, sp.unused24, 
			sp.unused25, sp.unused26);
		fprintf(iopo, ICS, sp.leven, sp.lpspol, sp.lovrok, sp.lcalda, 
			sp.unused27);
		fprintf(iopo, CCS2, sp.kstnm, sp.kevnm);
		fprintf(iopo, CCS1, sp.khole, sp.ko, sp.ka);
		fprintf(iopo, CCS1, sp.kt0, sp.kt1, sp.kt2);
		fprintf(iopo, CCS1, sp.kt3, sp.kt4, sp.kt5);
		fprintf(iopo, CCS1, sp.kt6, sp.kt7, sp.kt8);
		fprintf(iopo, CCS1, sp.kt9, sp.kf, sp.kuser0);
		fprintf(iopo, CCS1, sp.kuser1, sp.kuser2, sp.kcmpnm);
		fprintf(iopo, CCS1, sp.knetwk, sp.kdatrd, sp.kinst);
										*/
	/* print data in ascii */
		for (j=0; j<ns; j++) 
		{
			fprintf(iopo,"%15.7e", ar[j]);
			if (j != 0 && (j+1)%5 == 0) fprintf(iopo, "\n");
		}
		fprintf(iopo, "\n\n");
	}
	
/*    close file      */
	fclose (iopo) ;

}
