/*
	Program SEGY2sac
        
        Takes the output of any porogram that produces SEGY files
	and reformats them into a SAC file.


	The SEGY modification is as follows:
			trhdr[100]= MSB of sample rate in microseconds
			trhdr[101]  LSB of sample rate in microseconds
			trhdr[102]=1 ==> 32 bit integer data
			trhdr[102]=0 ==> 16 bit integer data
			trhdr[103]  milliseconds of start of trace
			trhdr[104]  trigger time year
			trhdr[105]  trigger time julian day
			trhdr[106]  trigger time hour
			trhdr[107]  trigger time minute
			trhdr[108]  trigger time second
			trhdr[109]  trigger time millisecond
			trhdr[110]  float scale factor (first half)
			trhdr[111]  scale factor (second half)


	Written by Tim Ahern, IRIS
		June 10, 1988

	modified by Tim Ahern, IRIS
		8/17/88
		to make use of trhdr[103] 
		   transfer more information into SAC header
		   loop on argv to pick up multiple files

	modified by Tim Ahern, IRIS
		8/31/88
		to put event trigger times into sac variable "a"

	modified by Jim Fowler, IRIS
		10/28/88
		to scale data to voltage and an option for no scale
		option -g on command line before file name means do
		not apply scale factor to data.  Allows you to look
		at counts into the D/A.

*/


#include <stdio.h>	/* Standard I/O header file			*/
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include "sac.h"

short	trhdr[120];

main (argc, argv)


int		argc;
char		*argv[];

{
   FILE	*fpin;			/* Pointer to the input file  */

   
   int i, j, int_hold, num_sam, sam_rate;
   short short_hold;
   unsigned char *cptr, *cptr1;
   unsigned char FileIN[64], FileOUT[64] ;
   float dt, ar[50000]; 
   int ifile;
    float scale ;
    int gain, sclflg ;

    /* check options flags */
    sclflg = 0 ;   /* default to scale data */
    while (argc > 1 && argv[1][0] == '-')
    {
	switch (argv[1][1])
	{
	    case 'g' :  /* -g no gain to be applied */
		sclflg = 1 ;
		break ;
	}
	argc-- ;
	argv++ ;
    }

   for (i=0 ; i<64 ; i++) FileIN[i] = 0;
   if(argc == 1){
      printf("\ Usage %s [options] file_name(s) \n", argv[0]);
      exit(1);
   }
for (ifile =1; ifile <argc; ifile++){
   strcpy( FileIN, &argv[ifile][0] );
   if( strcmp(FileIN,"sac") == 0)continue; /* don't work on a previous directORY */
   if( strcmp(FileIN,"ah") == 0)continue; /* don't work on a previous directORY */
   printf(" %s \n",FileIN);

         /* open the input file and go to the output directory	*/

   if (( fpin = fopen(FileIN, "rb")) == NULL ){
      printf( "\n ******  there has been an error opening %s for input  *****", FileIN);
      fclose(fpin);
      exit(1);
   }

   /*  start reading the data file   */
   /*  first read the SEGY trace header    */

   cptr = ( unsigned char *) &trhdr[0];
   for ( i= 0; i< 240 ; i++){
      *cptr = fgetc(fpin);
      cptr++;
   }
   num_sam = trhdr[57];
   if (trhdr[58] == 1){
      sam_rate = trhdr[100] * 32768 + trhdr[101];
   }else{
      sam_rate = trhdr[58];
   }
   dt = ( float) 1000000.0 / sam_rate ;

/*  get scale factor for data  */
    scale = 1 ;    /* default scale factor */
    if( sclflg == 0)
    {
	gain = trhdr[60] ;     /* get amplifier gain */
	cptr = (unsigned char *)&trhdr[110] ;  /* scale factor */
	cptr1 = (unsigned char *)&scale ;
	for( i=0 ; i<4 ; i++,cptr++,cptr1++) *cptr1 = *cptr ;
	if((gain == 0) || (scale == 0.))
	{
	    printf("\nERROR in scale factor gain = %d scale = %e",gain,scale);
	    printf("   Scale set to 1") ;
	    scale = 1. ;
	}
	else
	{
	    scale = scale/gain ;
	}
    }

   /* now go get the trace data */ 
   if (trhdr[102] == 1){            /* 32 bit integer data */

      for ( i=0; i<num_sam; i++){
         cptr = (unsigned char *) &int_hold;
         for (j=0; j<4; j++,cptr++){ 
            *cptr = fgetc(fpin);
         }
         ar[i] = ((float) int_hold)*scale;
      }
   }else{                         /* 16 bit integer data */
      for ( i=0; i < num_sam; i++){
         cptr = (unsigned char *) &short_hold;
         *cptr = fgetc(fpin);
	 cptr++;
         *cptr = fgetc(fpin);
	 ar[i] = ((float) short_hold)*scale ;
      }
   }

/*  prepend the ./sac directory name   */
   strcpy(FileOUT,"./sac/");
   strcat(FileOUT,FileIN);
/* make the output sac directory if needed    */
   mkdir("./sac",0777);
   wrtsac(FileOUT,dt,num_sam,ar,0);
   fclose(fpin);
}
}
   


/*********************************************************************/
/*                   wrtsac (FO,dt,ns,ar,flg)                        */
/*********************************************************************/
/*          Routine to write SAC files                               */
/*		FO = output file name                                */
/*		dt = sample rate in samples per second               */
/*		ns = number of data samples                          */
/*		ar = data array                                      */
/*		flg = output flag                                    */
/*		    = 0 output in binary format                      */
/*		    = 1 output in asci format                        */
/*                                                                   */
/*	    Program will add ".sac" or ".asc" extension to output    */
/*	    File name					             */

wrtsac (FO,dt,ns,ar,flg)
char	FO[] ;
int	ns, flg ;
float	dt, ar[] ;
{
    
        struct  sac	sp ;
	int	j, n0 ;
	float	zero = 0.0 ;
	float   t_beg, t_event, t_calc;

	FILE	*iopo ;


	if (flg == 0)
	{
	    strcat (FO,".sac") ;
	}
	else
	{
	    strcat (FO,".asc") ;
	}
	if((iopo = fopen(FO,"w")) == NULL)
	{
	    printf("cannot open output file %s\n",FO) ;
	    exit (1) ;
	}
    

/* convert parameters to sac header parameters */
	sp = sac_null;

/* floats */
	sp.delta = 1.0/dt ;
	sp.b = 0.0 ;
	sp.e = ns * sp.delta ;

/* integers */
	sp.internal4 = 6;	/* LLL sets it */
	sp.internal5 = 0;	/* LLL sets it */
	sp.internal6 = 0;	/* LLL sets it */
	sp.npts = ns ;
	sp.iftype = ITIME ;
	sp.iztype = IB ;

/* logicals  */
	sp.leven = TRUE ;
	sp.lpspol = FALSE ;     /* LLL sets it */
	sp.lcalda = TRUE ;      /* LLL sets it */
	sp.unused27 = FALSE ;   /* LLL sets it */

/* add some SEGY stuff   */
/* 				first some time information */

	sp.nzyear = trhdr[78];	/* the year */
	sp.nzjday = trhdr[79];	/* the julian day */	
	sp.nzhour = trhdr[80];	/* the hour */	
	sp.nzmin = trhdr[81];		/* the minute */	
	sp.nzsec = trhdr[82];		/* the second */	
	sp.nzmsec = trhdr[103];	/* the milliseconds */	

/*				get the trigger time now  */
	t_beg = trhdr[80]* 3600.0 + trhdr[81]*60.0 + (float) trhdr[82] + (float) trhdr[103]/ 1000.0;
	t_event = trhdr[106]* 3600.0 + trhdr[107]*60.0 + (float) trhdr[108] + (float) trhdr[109]/ 1000.0;
	t_calc = t_event - t_beg;
	if(t_calc < 0){ t_calc += 24.0 * 3600.0;}      /*handle the jump over a day   */   
	sp.a = t_calc;

/* 				now some coordinate information   */
	if(trhdr[40] != 0 && trhdr[41] != 0){
	   sp.stla = (float) ( (trhdr[40] << 16 ) | trhdr[41])/3600;
	   sp.stlo = (float) ( (trhdr[42] << 16 ) | trhdr[43])/3600;
 	}
	if (flg == 0) 
	{

	/* write binary sac header */
		if (fwrite(&sp, sizeof(struct sac), 1, iopo)!=1) 
		{
			printf("FATAL ERROR: bad  write %s, header\n",FO);
			exit(4);
		}

	/* write floating point data */
		if (fwrite(&ar[0],ns*sizeof(float),1,iopo)!=1) 
		{
			printf("FATAL ERROR: bad  write %s, data\n",FO);
			exit(5);
		}

	/* zero pad to 512 byte blocks (128 4 byte floats) */
		n0 = 128-(sizeof(struct sac)/4+ns)%128;
		for (j=0; j<n0; j++) 
		{
			if (fwrite(&zero, sizeof(float), 1, iopo)!=1) 
			{
				printf("FATAL ERROR: bad  write %s, padding\n"
					,FO);
				exit(6);
			}
		}
	}
    	else 
	{

	/* print header in ascii */
		fprintf(iopo, FCS, sp.delta, sp.depmin, sp.depmax, sp.scale, 
			sp.odelta);
		fprintf(iopo, FCS, sp.b, sp.e, sp.o, sp.a, sp.internal1);
		fprintf(iopo, FCS, sp.t0, sp.t1, sp.t2, sp.t3, sp.t4);
		fprintf(iopo, FCS, sp.t5, sp.t6, sp.t7, sp.t8, sp.t9);
		fprintf(iopo, FCS, sp.f, sp.resp0, sp.resp1, sp.resp2, 
			sp.resp3);
		fprintf(iopo, FCS, sp.resp4, sp.resp5, sp.resp6, sp.resp7, 
			sp.resp8);
		fprintf(iopo, FCS, sp.resp9, sp.stla, sp.stlo, sp.stel, 
			sp.stdp);
		fprintf(iopo, FCS, sp.evla, sp.evlo, sp.evel, sp.evdp, 
			sp.unused1);
		fprintf(iopo, FCS, sp.user0, sp.user1, sp.user2, sp.user3, 
			sp.user4);
		fprintf(iopo, FCS, sp.user5, sp.user6, sp.user7, sp.user8, 
			sp.user9);
		fprintf(iopo, FCS, sp.dist, sp.az, sp.baz, sp.gcarc, 
			sp.internal2);
		fprintf(iopo, FCS, sp.internal3, sp.depmen, sp.cmpaz, 
			sp.cmpinc, sp.unused2);
		fprintf(iopo, FCS, sp.unused3, sp.unused4, sp.unused5, 
			sp.unused6, sp.unused7);
		fprintf(iopo, FCS, sp.unused8, sp.unused9, sp.unused10, 
			sp.unused11, sp.unused12);
		fprintf(iopo, ICS, sp.nzyear, sp.nzjday, sp.nzhour, sp.nzmin, 
			sp.nzsec);
		fprintf(iopo, ICS, sp.nzmsec, sp.internal4, sp.internal5, 
			sp.internal6, sp.npts);
		fprintf(iopo, ICS, sp.internal7, sp.internal8, sp.unused13, 
			sp.unused14, sp.unused15);
		fprintf(iopo, ICS, sp.iftype, sp.idep, sp.iztype, sp.unused16,
			sp.iinst);
		fprintf(iopo, ICS, sp.istreg, sp.ievreg, sp.ievtyp, sp.iqual, 
			sp.isynth);
		fprintf(iopo, ICS, sp.unused17, sp.unused18, sp.unused19, 
			sp.unused20, sp.unused21);
		fprintf(iopo, ICS, sp.unused22, sp.unused23, sp.unused24, 
			sp.unused25, sp.unused26);
		fprintf(iopo, ICS, sp.leven, sp.lpspol, sp.lovrok, sp.lcalda, 
			sp.unused27);
		fprintf(iopo, CCS2, sp.kstnm, sp.kevnm);
		fprintf(iopo, CCS1, sp.khole, sp.ko, sp.ka);
		fprintf(iopo, CCS1, sp.kt0, sp.kt1, sp.kt2);
		fprintf(iopo, CCS1, sp.kt3, sp.kt4, sp.kt5);
		fprintf(iopo, CCS1, sp.kt6, sp.kt7, sp.kt8);
		fprintf(iopo, CCS1, sp.kt9, sp.kf, sp.kuser0);
		fprintf(iopo, CCS1, sp.kuser1, sp.kuser2, sp.kcmpnm);
		fprintf(iopo, CCS1, sp.knetwk, sp.kdatrd, sp.kinst);

	/* print data in ascii */
		for (j=0; j<ns; j++) 
		{
			fprintf(iopo,"%15.7e", ar[j]);
			if (j != 0 && (j+1)%5 == 0) fprintf(iopo, "\n");
		}
		fprintf(iopo, "\n\n");
	}
	
/*    close file      */
	fclose (iopo) ;

}
